param([switch]$InstallTask, [switch]$UninstallTask)
#requires runasadministrator

Class EmbyServiceUpdater
{

    $location = $this.getlocation()
    $releaseChannel = $this.getReleaseChannel()
    $release = $this.getLatestRelease()

    [string]getlocation() {
        Write-Progress "Getting Install Location"
        Start-Sleep 1
        return (Get-Item (Get-ItemProperty "HKLM:\SYSTEM\CurrentControlSet\Services\Emby").`
        ImagePath.split("`"")[1]).Directory.Parent.FullName
    }

    [object]getLatestRelease() {
        Write-Progress "Getting Latest Release"
        $releases = ConvertFrom-Json (Invoke-WebRequest "https://api.github.com/repos/mediabrowser/emby/releases" -UseBasicParsing)
        if ($this.ReleaseChannel -eq "Release") {
            return ($releases | Where-Object {$_.prerelease -eq $true})[0]
        } else {
            return ($releases | Where-Object {$_.name -match $this.releaseChannel})[0]
        }
    }

    [version]getLocalVersion() {
        Write-Progress "Getting Local Version"
        Start-Sleep 1
        Push-Location $this.location
        $fileVersion = (Get-Item ".\system\MediaBrowser.ServerApplication.exe").VersionInfo.FileVersion
        Pop-Location
        return $fileVersion
    }
   
    [version]getRemoteVersion() {
        Write-Progress "Getting Remote Version"
        Start-Sleep 1
        return $this.release.tag_name
    }

    [string]getReleaseChannel() {
        Write-Progress "Getting Release Channel"
        Start-Sleep 1
        Push-Location $this.location
        [xml]$channel = Get-Content ".\config\system.xml"
        Pop-Location
        return $channel.ServerConfiguration.SystemUpdateLevel
    }

    [bool]isUpdate() {
        $remote = $this.getRemoteVersion()
        $local = $this.getLocalVersion()
        Write-Progress "Comparing Versions"
        Start-Sleep 1
        return $remote -gt $local
    }

    [string]getAssetUrl() {
        Write-Progress "Getting Download Location"
        Start-Sleep 1
        return ($this.release.assets | Where-Object {$_.name -eq "emby.windows.zip"}).browser_download_url
    }

    [void]getUpdate() {
        if (Test-Path $this.location) {
            Push-Location $this.location
            try {
                if ($this.isUpdate()) {
                    -not (test-path ".\updates") -and (mkdir ".\updates")
                    $asset = $this.getAssetUrl()
                    Write-Progress ("Downloading Update {0}" -f $this.release.tag_name)
                    Invoke-WebRequest $asset -OutFile ".\updates\MBserver.zip" -UseBasicParsing
                    $this.release.tag_name > ".\updates\MBserver.zip.ver"
                } else {
                    Write-Progress ("Latest Version {0}" -f $this.release.tag_name)
                    Start-Sleep 1
                }
            } catch {
               (Test-Path ".\updates") -and (Remove-Item ".\updates" -Recurse)
            }
            Pop-Location
        }
    }

    [void]install() {
        Push-Location $this.location
        if (Test-Path ".\Updates\MBserver.zip") {
            Write-Progress "Installing Update"
            Stop-Service Emby
            Wait-Process MediaBrowser.ServerApplication -ErrorAction SilentlyContinue
            (Test-Path ".\System.old") -and (Remove-Item ".\System.old" -Recurse)
            (Test-Path ".\System") -and (Move-Item ".\System" ".\System.old")
            -not (Test-Path ".\System") -and (Expand-Archive ".\Updates\MBserver.zip" ".")
            (Test-Path ".\System") -and (Remove-Item ".\updates" -Recurse)
            Write-Progress "Update Installed" -Completed
            Start-Service Emby
        }
        Pop-Location
    }

    [void]installTask($scriptName) {
        Write-Progress "Installing Task"
        Start-Sleep 1
        Push-Location $this.location
        -not (Test-Path ".\updater") -and (mkdir ".\updater")
        Copy-Item ("{0}\{1}" -f $PSScriptRoot, $scriptName) ".\updater\WindowsServiceUpdater.ps1" -Force
        Pop-Location
        $action = New-ScheduledTaskAction -Execute "Powershell.exe" `
        -Argument ("`"{0}\updater\WindowsServiceUpdater.ps1`"" -f $this.location)
        $trigger =  New-ScheduledTaskTrigger -Daily -At 4am
        Register-ScheduledTask -Action $action -Trigger $trigger -TaskName "Emby Service Updater" `
        -Description "Emby Service Updater" -User "SYSTEM" -Force
    }

    [void]uninstallTask() {
        Write-Progress "Uninstalling Task"
        Start-Sleep 1
        Push-Location $this.location
        (Test-Path ".\updater") -and (Remove-Item ".\updater" -Recurse)
        Pop-Location
        Unregister-ScheduledTask -TaskName "Emby Service Updater" -Confirm:$false
    }
}

$Updater = [EmbyServiceUpdater]::new()
if ($InstallTask){
    $Updater.installTask($MyInvocation.MyCommand.Name)

} elseif ($UninstallTask){
    $Updater.uninstallTask()

} else {
    $Updater.getUpdate()
    $Updater.install()
}